﻿using System;
using System.Collections.Generic;
using CellarDoor.Gui.Elements;
using Microsoft.Xna.Framework;

namespace CellarDoor.Gui.Layout
{
    public class UiLayout
    {
        /// <summary>
        /// Gets and sets a value indicating whether the layout must be updated or not.
        /// </summary>
        public bool Invalid { get; set; }

        private Vector2 size;
        /// <summary>
        /// Gets and sets the size of laid out element
        /// </summary>
        public Vector2 Size
        {
            get { return size; }
            set 
            {
                if (size != value)
                {
                    size = value;
                    Invalid = true;
                }
            }
        }
        private Vector2 sizeScaled;
        /// <summary>
        /// Gets the scaled size of the laid out element to the target resolution
        /// </summary>
        public Vector2 SizeScaled
        {
            get { return sizeScaled; }
            protected set { sizeScaled = value; }
        }

        private Vector2 location;
        /// <summary>
        /// Gets and sets the location of the laid aout element
        /// </summary>
        public Vector2 Location
        {
            get { return location; }
            set
            {
                if (location != value)
                {
                    location = value;
                    Invalid = true;
                }
            }
        }
        
        /// <summary>
        /// Gets the scaled location of the laid out element to the target resolution
        /// </summary>
        public Vector2 LocationScaled { get; protected set; }

        /// <summary>
        /// Gets the position of the laid out element to the target resolution
        /// </summary>
        public Vector2 Position { get; protected set; }

        /// <summary>
        /// Gets the scaled position of the laid out element to the target resolution
        /// </summary>
        public Vector2 PositionScaled { get; protected set; }

        private UiPadding padding;
        /// <summary>
        /// Gets and sets the padding of the laid out element.
        /// </summary>
        public UiPadding Padding
        {
            get { return padding; }
            set
            {
                if (padding != value)
                {
                    padding = value;
                    Invalid = true;
                }
            }
        }
        /// <summary>
        /// Gets the scaled padding of the laid out element to the target resolution.
        /// </summary>
        public UiPadding PaddingScaled { get; protected set; }

        private UiPadding margin;
        /// <summary>
        /// Gets and sets the margin of the laid out element.
        /// </summary>
        public UiPadding Margin
        {
            get { return margin; }
            set
            {
                if (margin != value)
                {
                    margin = value;
                    Invalid = true;
                }
            }
        }
        /// <summary>
        /// Gets the scaled margin of the laid out element to the target resolution.
        /// </summary>
        public UiPadding MarginScaled { get; protected set; }

        /// <summary>
        /// Gets the the text size of the laid out element.
        /// </summary>
        public Vector2 TextSize { get; protected set; }

        /// <summary>
        /// Gets the scaled text size of the laid out element to the target resolution.
        /// </summary>
        public Vector2 TextSizeScaled { get; protected set; }

        private Vector2 textLocation;
        /// <summary>
        /// Gets and sets the text location of the laid out element.
        /// </summary>
        public Vector2 TextLocation
        {
            get { return textLocation; }
            set
            {
                if (textLocation != value)
                {
                    textLocation = value;
                    Invalid = true;
                }
            }
        }
        /// <summary>
        /// Gets the scaled text location of the laid out element to the target resolution.
        /// </summary>
        public Vector2 TextLocationScaled { get; protected set; }

        /// <summary>
        /// Gets the text position of the laid out element.
        /// </summary>
        public Vector2 TextPosition { get; protected set; }

        /// <summary>
        /// Gets the scaled text position of the laid out element to the target resolution.
        /// </summary>
        public Vector2 TextPositionScaled { get; protected set; }

        private UiAutoSizeStyle autosize;
        /// <summary>
        /// Gets and sets the autosize property of the laid out element.
        /// </summary>
        public UiAutoSizeStyle Autosize
        {
            get { return autosize; }
            set
            {
                if (autosize != value)
                {
                    autosize = value;
                    Invalid = true;
                }
            }
        }

        private UiAlignStyle align;
        /// <summary>
        /// Gets and sets the align property of the laid out element.
        /// </summary>
        public UiAlignStyle Align
        {
            get { return align; }
            set
            {
                if (align != value)
                {
                    align = value;
                    Invalid = true;
                }
            }
        }
        private UiDockStyles dock;
        /// <summary>
        /// Gets and sets the dock property of the laid out element.
        /// </summary>
        public UiDockStyles Dock
        {
            get { return dock; }
            set
            {
                if (dock != value)
                {
                    dock = value;
                    Invalid = true;
                }
            }
        }
        private UiAlignStyle textAlign;
        /// <summary>
        /// Gets and sets the text align of the laid out element.
        /// </summary>
        public UiAlignStyle TextAlign
        {
            get { return textAlign; }
            set
            {
                if (textAlign != value)
                {
                    textAlign = value;
                    Invalid = true;
                }
            }
        }

        private UiScaleAxis sizeScaleAxis;
        /// <summary>
        /// Gets and sets the axis that is used to scale the size of the laid out element.
        /// </summary>
        public UiScaleAxis SizeScaleAxis
        {
            get { return sizeScaleAxis; }
            set
            {
                if (sizeScaleAxis != value)
                {
                    sizeScaleAxis = value;
                    Invalid = true;
                }
            }
        }
        private UiScaleAxis positionScaleAxis;
        /// <summary>
        /// Gets and sets the axis that is used to scale the position and location of the laid out element.
        /// </summary>
        public UiScaleAxis PositionScaleAxis
        {
            get { return positionScaleAxis; }
            set
            {
                if (positionScaleAxis != value)
                {
                    positionScaleAxis = value;
                    Invalid = true;
                }
            }
        }
        private UiScaleAxis paddingScaleAxis;
        /// <summary>
        /// Gets and sets the axis that is used to scale the padding and margin of the laid out element.
        /// </summary>
        public UiScaleAxis PaddingScaleAxis
        {
            get { return paddingScaleAxis; }
            set
            {
                if (paddingScaleAxis != value)
                {
                    paddingScaleAxis = value;
                    Invalid = true;
                }
            }
        }
        private UiScaleAxis textScaleAxis;
        /// <summary>
        /// Gets and sets the axis that is used to scale the text of the laid out element.
        /// </summary>
        public UiScaleAxis TextScaleAxis
        {
            get { return textScaleAxis; }
            set
            {
                if (textScaleAxis != value)
                {
                    textScaleAxis = value;
                    Invalid = true;
                }
            }
        }

        /// <summary>
        /// Gets the area that is covered of the laid out element.
        /// </summary>
        public Rectangle Area
        {
            get
            {
                return new Rectangle((int)Position.X, (int)Position.Y, (int)Size.X, (int)Size.Y);
            }
        }

        /// <summary>
        /// Gets the scaled area that is covered of the laid out element in the target resolution.
        /// </summary>
        public Rectangle AreaScaled
        {
            get
            {
                return new Rectangle((int)PositionScaled.X, (int)PositionScaled.Y, (int)SizeScaled.X, (int)SizeScaled.Y);
            }
        }

        /// <summary>
        /// Gets the laid out element
        /// </summary>
        public UiElement Element { get; private set; }

        private Dictionary<UiElement, RectangleF> dockingAreas = new Dictionary<UiElement, RectangleF>();
        private Dictionary<UiElement, RectangleF> dockingAreasScaled = new Dictionary<UiElement, RectangleF>();

        public UiLayout(UiElement element)
        {
            SizeScaleAxis = UiScaleAxis.Both;
            PositionScaleAxis = UiScaleAxis.Both;
            PaddingScaleAxis = UiScaleAxis.None;
            TextScaleAxis = UiScaleAxis.None;
            TextAlign = UiAlignStyle.Middle;
            Align = UiAlignStyle.None;
            Dock = UiDockStyles.None;
            Invalid = true;

            this.Element = element;
            this.Element.Elements.ControlAdded += Elements_ControlAdded;
            this.Element.Elements.ControlRemoved += Elements_ControlRemoved;
        }

        void Elements_ControlRemoved(object sender, UiElementEventArgs e)
        {
            this.Invalid = true;
            if (e.Element != null)
            {
                e.Element.BoundsChanged -= Element_BoundsChanged;
            }
        }

        void Elements_ControlAdded(object sender, UiElementEventArgs e)
        {
            this.Invalid = true;
            if (e.Element != null)
            {
                e.Element.BoundsChanged += Element_BoundsChanged;
            }
        }

        void Element_BoundsChanged(object sender, UiEventArgs args)
        {
            this.Invalid = true;
        }

        public void Update()
        {
            if (this.Autosize != UiAutoSizeStyle.None)
            {
                UpdateAutosizeLayout();
            }
            else
            {
                ScaleSizes(Element.Layout, Element.Manager);
            }

            if (Element.Parent != null)
            {
                if (this.Dock != UiDockStyles.None)
                {
                    UpdateDockLayout();
                }
                else if (this.Align != UiAlignStyle.None)
                {
                    UpdateAlignLayout();
                }
                else
                {
                    UpdateDefaultLayout();
                }
            }
            else
            {
                UpdateDefaultLayout();
            }

            UpdateDockedChildren();
            UpdateTextLayout();
        }

        private void UpdateAutosizeLayout()
        {
            float finalWidth = 0;
            float finalHeight = 0;

            foreach (var element in Element.Elements)
            {
                if (element.Layout.Align != UiAlignStyle.None)
                {
                    continue;
                }

                float width = finalWidth;
                float height = finalHeight;

                switch (element.Layout.Dock)
                {
                    case UiDockStyles.None:
                        height = element.Layout.Location.X + element.Layout.Size.X;
                        width = element.Layout.Location.X + element.Layout.Size.X;
                        break;
                    case UiDockStyles.Top:
                    case UiDockStyles.Bottom:
                        height = element.Layout.Location.X + element.Layout.Size.X;
                        break;
                    case UiDockStyles.Left:
                    case UiDockStyles.Right:
                        width = element.Layout.Location.X + element.Layout.Size.X;
                        break;
                    case UiDockStyles.Fill:
                        break;
                    default:
                        break;
                }

                if (width > finalWidth)
                {
                    finalWidth = width;
                }
                if (height > finalHeight)
                {
                    finalHeight = height;
                }
            }

            switch (Autosize)
            {
                case UiAutoSizeStyle.None:
                    break;
                case UiAutoSizeStyle.GrowOnly:
                    if (size.X < finalWidth)
                    {
                        size.X = finalWidth;
                    }
                    if (size.Y < finalHeight)
                    {
                        size.Y = finalHeight;
                    }
                    break;
                case UiAutoSizeStyle.GrowAndShrink:
                    Size = new Vector2(finalWidth, finalHeight);
                    break;
                default:
                    break;
            }

            ScaleSizes(this, Element.Manager);
        }

        protected static void ScaleSizes(UiLayout layout, UiManager manager)
        {
            layout.PaddingScaled = manager.GetScaledPadding(layout.PaddingScaleAxis, layout.Padding);
            layout.MarginScaled = manager.GetScaledPadding(layout.PaddingScaleAxis, layout.Margin);
            layout.SizeScaled = layout.Size * manager.GetScaleFactor(layout.SizeScaleAxis);
        }

        protected virtual void UpdateDefaultLayout()
        {
            this.LocationScaled = this.Location * Element.Manager.GetScaleFactor(this.PositionScaleAxis);
            if (Element.Parent != null)
            {
                this.Position = Element.Parent.Layout.Position + this.Location;
                this.PositionScaled = Element.Parent.Layout.PositionScaled + this.LocationScaled;
            }
            else
            {
                this.Position = this.Location;
                this.PositionScaled = this.LocationScaled;
            }
        }

        protected virtual void UpdateTextLayout()
        {
            if (!String.IsNullOrEmpty(Element.Text) && Element.Style.Font != null)
            {
                this.TextSize = Element.Style.Font.MeasureString(Element.Text);
                this.TextLocation = UiLayoutHelper.AlignItem(this.TextAlign,
                    this.Size, this.Padding, this.Margin,
                    this.TextSize, UiPadding.Zero, UiPadding.Zero);
                this.TextPosition = this.Position + this.TextLocation;

                this.TextSizeScaled = Element.Manager.GetScaleFactor(this.TextScaleAxis) * this.TextSize;
                this.TextLocationScaled = UiLayoutHelper.AlignItem(this.TextAlign,
                    this.SizeScaled, this.PaddingScaled, this.MarginScaled,
                    this.TextSizeScaled, UiPadding.Zero, UiPadding.Zero);
                this.TextPositionScaled = this.PositionScaled + this.TextLocationScaled;
            }
        }

        protected virtual void UpdateAlignLayout()
        {
            if (Element.Parent == null)
            {
                throw new InvalidOperationException("Element must have a parent to be aligned");
            }

            UiLayout parent = Element.Parent.Layout;
            UiManager manager = Element.Manager;

            this.Location = UiLayoutHelper.AlignItem(
                this.Align,
                parent.Size, parent.Padding, parent.Margin,
                this.Size, this.Padding, this.Margin);

            this.LocationScaled = UiLayoutHelper.AlignItem(
                this.Align,
                parent.SizeScaled,
                manager.GetScaledPadding(parent.SizeScaleAxis, parent.Padding),
                manager.GetScaledPadding(parent.SizeScaleAxis, parent.Margin),
                this.SizeScaled,
                manager.GetScaledPadding(this.SizeScaleAxis, this.Padding),
                manager.GetScaledPadding(this.SizeScaleAxis, this.Margin));

            this.Position = parent.Position + this.Location;
            this.PositionScaled = parent.PositionScaled + this.LocationScaled;
        }

        protected virtual void UpdateDockLayout()
        {
            RectangleF area;
            if (Element.Parent.Layout.GetDockArea(Element, out area))
            {
                this.Location = new Vector2(area.X, area.Y);
                this.Size = new Vector2(area.Width, area.Height);
            }
            if (Element.Parent.Layout.GetDockAreaScaled(Element, out area))
            {
                this.LocationScaled = new Vector2(area.X, area.Y);
                this.SizeScaled = new Vector2(area.Width, area.Height);
            }

            this.Position = Element.Parent.Layout.Position + this.Location;
            this.PositionScaled = Element.Parent.Layout.PositionScaled + this.LocationScaled;
        }

        public bool GetDockArea(UiElement element, out RectangleF area)
        {
            return dockingAreas.TryGetValue(element, out area);
        }

        public bool GetDockAreaScaled(UiElement element, out RectangleF area)
        {
            return dockingAreasScaled.TryGetValue(element, out area);
        }

        protected virtual void UpdateDockedChildren()
        {
            RectangleF pArea = new RectangleF(Vector2.Zero, this.Size);
            RectangleF pAreaScaled = new RectangleF(Vector2.Zero, this.SizeScaled);

            UiPadding pPadding = this.Padding;
            UiPadding pPaddingScaled = this.PaddingScaled;

            dockingAreas.Clear();
            dockingAreasScaled.Clear();
            foreach (UiElement item in Element.Elements)
            {
                if (item.Dock == UiDockStyles.None)
                {
                    continue;
                }

                ScaleSizes(item.Layout, item.Manager);

                UiLayout layout = item.Layout;
                RectangleF cArea = new RectangleF(Vector2.Zero, layout.Size);
                RectangleF cAreaScaled = new RectangleF(Vector2.Zero, layout.SizeScaled);

                UiPadding cMargin = layout.Margin;
                UiPadding cMarginScaled = layout.MarginScaled;

                UiLayoutHelper.DockItem(item.Dock, ref pArea, ref pPadding, ref cArea, ref cMargin);
                UiLayoutHelper.DockItem(item.Dock, ref pAreaScaled, ref pPaddingScaled, ref cAreaScaled, ref cMarginScaled);

                dockingAreas.Add(item, cArea);
                dockingAreasScaled.Add(item, cAreaScaled);
            }
        }
    }
}
