﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using CellarDoor.Gui.Elements;

namespace CellarDoor.Gui.Layout
{
    public static class UiLayoutHelper
    {
        public static Vector2 GetScaleFactor(this UiManager manager, UiScaleAxis axis)
        {
            switch (axis)
            {
                case UiScaleAxis.X:
                    return new Vector2(manager.ElementScale.X);
                case UiScaleAxis.Y:
                    return new Vector2(manager.ElementScale.Y);
                case UiScaleAxis.Both:
                    return manager.ElementScale;
                case UiScaleAxis.None:
                    return Vector2.One;
                default:
                    return Vector2.One;
            }
        }

        public static UiPadding GetScaledPadding(this UiManager manager, UiScaleAxis axis, UiPadding padding)
        {
            Vector2 factor = GetScaleFactor(manager, axis);
            if (factor != Vector2.One)
            {
                return new UiPadding(
                    (int)(padding.Top * factor.Y),
                    (int)(padding.Bottom * factor.Y),
                    (int)(padding.Left * factor.X),
                    (int)(padding.Right * factor.X));
            }
            else
            {
                return padding;
            }
        }

        public static Vector2 AlignItem(UiAlignStyle align,
            Vector2 pSize, UiPadding pPadding, UiPadding pMargin,
            Vector2 cSize, UiPadding cPadding, UiPadding cMargin)
        {
            Vector2 location = Vector2.Zero;

            #region Horizontal Align
            switch (align)
            {
                case UiAlignStyle.Left:
                case UiAlignStyle.Left | UiAlignStyle.Top:
                case UiAlignStyle.Left | UiAlignStyle.Middle:
                case UiAlignStyle.Left | UiAlignStyle.Bottom:
                case UiAlignStyle.Left | UiAlignStyle.VerticalNone:
                case UiAlignStyle.Left | UiAlignStyle.OutsideTop:
                case UiAlignStyle.Left | UiAlignStyle.OutsideBottom:
                    location.X = Math.Max(pPadding.Left, cMargin.Left);
                    break;
                case UiAlignStyle.Right:
                case UiAlignStyle.Right | UiAlignStyle.Top:
                case UiAlignStyle.Right | UiAlignStyle.Middle:
                case UiAlignStyle.Right | UiAlignStyle.Bottom:
                case UiAlignStyle.Right | UiAlignStyle.VerticalNone:
                case UiAlignStyle.Right | UiAlignStyle.OutsideTop:
                case UiAlignStyle.Right | UiAlignStyle.OutsideBottom:
                    location.X = pSize.X - cSize.X - Math.Max(pPadding.Right, cMargin.Right);
                    break;
                case UiAlignStyle.Middle:
                case UiAlignStyle.Middle | UiAlignStyle.Top:
                case UiAlignStyle.Middle | UiAlignStyle.Bottom:
                case UiAlignStyle.Middle | UiAlignStyle.VerticalNone:
                case UiAlignStyle.Middle | UiAlignStyle.OutsideTop:
                case UiAlignStyle.Middle | UiAlignStyle.OutsideBottom:
                    location.X = (pSize.X - cSize.X) * 0.5f;
                    break;
                case UiAlignStyle.OutsideLeft:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.Top:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.Middle:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.Bottom:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.VerticalNone:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.OutsideTop:
                case UiAlignStyle.OutsideLeft | UiAlignStyle.OutsideBottom:
                    location.X = -(cSize.X + Math.Max(pMargin.Left, cMargin.Right));
                    break;
                case UiAlignStyle.OutsideRight:
                case UiAlignStyle.OutsideRight | UiAlignStyle.Top:
                case UiAlignStyle.OutsideRight | UiAlignStyle.Middle:
                case UiAlignStyle.OutsideRight | UiAlignStyle.Bottom:
                case UiAlignStyle.OutsideRight | UiAlignStyle.VerticalNone:
                case UiAlignStyle.OutsideRight | UiAlignStyle.OutsideTop:
                case UiAlignStyle.OutsideRight | UiAlignStyle.OutsideBottom:
                    location.X = pSize.X + Math.Max(pMargin.Right, cMargin.Left);
                    break;
            }
            #endregion

            #region Vertical Align
            switch (align)
            {
                case UiAlignStyle.Top:
                case UiAlignStyle.Top | UiAlignStyle.Left:
                case UiAlignStyle.Top | UiAlignStyle.Middle:
                case UiAlignStyle.Top | UiAlignStyle.Right:
                case UiAlignStyle.Top | UiAlignStyle.HorizontalNone:
                case UiAlignStyle.Top | UiAlignStyle.OutsideLeft:
                case UiAlignStyle.Top | UiAlignStyle.OutsideRight:
                    location.Y = Math.Max(pPadding.Top, cMargin.Top);
                    break;
                case UiAlignStyle.Bottom:
                case UiAlignStyle.Bottom | UiAlignStyle.Left:
                case UiAlignStyle.Bottom | UiAlignStyle.Middle:
                case UiAlignStyle.Bottom | UiAlignStyle.Right:
                case UiAlignStyle.Bottom | UiAlignStyle.HorizontalNone:
                case UiAlignStyle.Bottom | UiAlignStyle.OutsideLeft:
                case UiAlignStyle.Bottom | UiAlignStyle.OutsideRight:
                    location.Y = pSize.Y - cSize.Y - Math.Max(pPadding.Bottom, cMargin.Bottom);
                    break;
                case UiAlignStyle.Middle:
                case UiAlignStyle.Middle | UiAlignStyle.Left:
                case UiAlignStyle.Middle | UiAlignStyle.Right:
                case UiAlignStyle.Middle | UiAlignStyle.HorizontalNone:
                case UiAlignStyle.Middle | UiAlignStyle.OutsideLeft:
                case UiAlignStyle.Middle | UiAlignStyle.OutsideRight:
                    location.Y = (pSize.Y - cSize.Y) * 0.5f;
                    break;
                case UiAlignStyle.OutsideTop:
                case UiAlignStyle.OutsideTop | UiAlignStyle.Left:
                case UiAlignStyle.OutsideTop | UiAlignStyle.Middle:
                case UiAlignStyle.OutsideTop | UiAlignStyle.Right:
                case UiAlignStyle.OutsideTop | UiAlignStyle.HorizontalNone:
                case UiAlignStyle.OutsideTop | UiAlignStyle.OutsideLeft:
                case UiAlignStyle.OutsideTop | UiAlignStyle.OutsideRight:
                    location.Y = -cSize.Y - Math.Max(pMargin.Top, cMargin.Bottom);
                    break;
                case UiAlignStyle.OutsideBottom:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.Left:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.Middle:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.Right:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.HorizontalNone:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.OutsideLeft:
                case UiAlignStyle.OutsideBottom | UiAlignStyle.OutsideRight:
                    location.Y = pSize.Y + Math.Max(pMargin.Bottom, cMargin.Top);
                    break;
            }
            #endregion

            return location;
        }

        public static void DockItem(UiDockStyles dockStyle,
            ref RectangleF pArea, ref UiPadding pPadding,
            ref RectangleF cArea, ref UiPadding cMargin)
        {
            UiPadding pad = cMargin.MaxMerge(pPadding);
            switch (dockStyle)
            {
                case UiDockStyles.Top:
                    cArea.Y = pArea.Top + pad.Top;
                    cArea.X = pArea.Left + pad.Left;
                    cArea.Width = pArea.Width - (pad.Left + pad.Right);

                    pArea.Y = cArea.Bottom;
                    pArea.Height = pArea.Height - cArea.Height - pad.Top;
                    pPadding.Top = cMargin.Bottom;
                    break;
                case UiDockStyles.Bottom:
                    cArea.Y = pArea.Bottom - cArea.Height - pad.Bottom;
                    cArea.X = pArea.Left + pad.Left;
                    cArea.Width = pArea.Width - (pad.Left + pad.Right);

                    pArea.Height = pArea.Height - cArea.Height - pad.Bottom;
                    pPadding.Bottom = cMargin.Top;
                    break;
                case UiDockStyles.Left:
                    cArea.Y = pArea.Top + pad.Top;
                    cArea.X = pArea.Left + pad.Left;
                    cArea.Height = pArea.Height - (pad.Top + pad.Bottom);

                    pArea.X = cArea.Right;
                    pArea.Width = pArea.Width - cArea.Width - pad.Left;
                    pPadding.Left = cMargin.Right;
                    break;
                case UiDockStyles.Right:
                    cArea.Y = pArea.Top + pad.Top;
                    cArea.X = pArea.Right - cArea.Width - pad.Right;
                    cArea.Height = pArea.Height - (pad.Top + pad.Bottom);

                    pArea.Width = pArea.Width - cArea.Width - pad.Right;
                    pPadding.Right = cMargin.Left;
                    break;
                case UiDockStyles.Fill:
                    cArea.Y = pArea.Top + pad.Top;
                    cArea.X = pArea.Left + pad.Left;
                    cArea.Height = pArea.Height - (pad.Top + pad.Bottom);
                    cArea.Width = pArea.Width - (pad.Left + pad.Right);

                    pArea.Width = 0;
                    pArea.Height = 0;
                    pPadding = new UiPadding(0);
                    break;
            }
        }
    }
}
