﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using CellarDoor.Gui.Controller;
using CellarDoor.Gui.Animation;
using CellarDoor.Gui.Theming;
using CellarDoor.Gui.Layout;

namespace CellarDoor.Gui.Layout
{
    public class UiVisualLayer
    {
        /// <summary>
        /// Gets and sets the sprite to render
        /// </summary>
        public UiThemeSprite Sprite { get; set; }

        private UiPadding extent;
        /// <summary>
        /// Gets and sets the area extension
        /// </summary>
        public UiPadding Extent
        {
            get { return extent; }
            set
            {
                if (extent != value)
                {
                    extent = value;
                    ExtendTargetArea();
                }
            }
        }

        /// <summary>
        /// Gets and sets a value indicating whether this layer is enable and should be drawn or not
        /// </summary>
        public bool Enabled { get; set; }

        /// <summary>
        /// Gets and sets the layer opacity
        /// </summary>
        public float  Opacity { get; set; }

        /// <summary>
        /// Gets and sets the layer tint color
        /// </summary>
        public Vector3 Tint { get; set; }

        private UiRectangle target;
        /// <summary>
        /// Gets and sets the target area where the sprite is drawn to
        /// </summary>
        public UiRectangle Target
        {
            get { return target; }
            set
            {
                if (target != value)
                {
                    target = value;
                    ExtendTargetArea();
                }
            }
        }

        private UiRectangle targetExtended;
        /// <summary>
        /// Gets the target area that is combined with the <see cref="Extent"/> property
        /// </summary>
        public UiRectangle TargetExtended 
        {
            get { return targetExtended; } 
        }

        public UiVisualLayer()
        {
            Enabled = true;
            Opacity = 1;
            Tint = Vector3.One;
        }

        public void SetTargetArea(Rectangle rectangle)
        {
            target.SetArea(rectangle, extent);
            ExtendTargetArea();
        }

        private void ExtendTargetArea()
        {
            if (Sprite != null)
            {
                targetExtended = Sprite.Rectangle;
                targetExtended.SetArea(target.GetArea(), extent);
            }
            else
            {
                targetExtended = target;
                targetExtended.SetArea(target.GetArea(), extent);
            }
        }
    }
}
