﻿using System;
using Microsoft.Xna.Framework;
using CellarDoor.Gui.Elements;
using CellarDoor.Gui.Controller;
using CellarDoor.Gui.Animation;
using CellarDoor.Gui.Theming;
using CellarDoor.Gui.Layout;

namespace CellarDoor.Gui.Layout
{
    public class UiVisualStyleWithState : UiVisualStyle
    {
        public UiVisualLayer DefaultState { get; protected set; }
        public UiVisualLayer HoverState { get; protected set; }
        public UiVisualLayer DownState { get; protected set; }
        public UiVisualLayer DisabledState { get; protected set; }

        public UiVisualStyleWithState(UiElement element)
            : base(element)
        {
            DefaultState = new UiVisualLayer();
            HoverState = new UiVisualLayer();
            DownState = new UiVisualLayer();
            DisabledState = new UiVisualLayer();
        }

        protected override void LoadTheme(UiTheme theme, int resX, int resY)
        {
            if (ElementName != null)
            {
                this.DefaultState.Sprite = theme.GetState(ElementName, "Default", resX, resY);
                this.HoverState.Sprite = theme.GetState(ElementName, "Hover", resX, resY);
                this.DownState.Sprite = theme.GetState(ElementName, "Down", resX, resY);
                this.DisabledState.Sprite = theme.GetState(ElementName, "Disabled", resX, resY);
            }

            base.LoadTheme(theme, resX, resY);
        }

        protected override void UpdateVisualState()
        {
            switch (State)
            {
                case UiVisualState.Default:
                    StateLayer = DefaultState;
                    break;
                case UiVisualState.Disabled:
                    StateLayer = DisabledState;
                    break;
                case UiVisualState.Up:
                    StateLayer = HoverState;
                    break;
                case UiVisualState.Down:
                    StateLayer = DownState;
                    break;
                default:
                    StateLayer = DefaultState;
                    break;
            }
        }

        public override void SetTargetArea(Rectangle area)
        {
            DefaultState.SetTargetArea(area);
            HoverState.SetTargetArea(area);
            DownState.SetTargetArea(area);
            DisabledState.SetTargetArea(area);

            base.SetTargetArea(area);
        }
    }
}
